      program simplefit
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! A program to superpose like-molecule assemblies of peptides/proteins !
! by the reference algorithm, which involves the enumeration of all    !
! mappings of the chains of the superposed on those of the reference   !
! assembly, and the simple algorithm that superposes chains with same  !
! ID. Molecules, 2025, submitted.                                      !
!                                                                      !
! Usage:                                                               !
! simplefit pdbfile.pdb                                                !
!                                                                      !
! pdbfile.pdb is the PDB file containing all structures to superpose   !
!              of which all pairs will be considered, chains separated !
!              with TER and structures separated with ENDMDL           !
!              (mandatory, otherwise the program will crash). The file !
!              must have the .pdb extension.                           !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      implicit none
      include "DIMENSIONS"
      include "sizesclu.dat"
      include "COMMON.CONTROL"
      include "COMMON.NAMES"
      include "COMMON.CHAIN"
      include "COMMON.CLUSTER"
      include "COMMON.IOUNITS"
      double precision rmsd_simple,rmsd
      integer i,j,k,icon,jcon,ii,ncon,iretcode,nfun,imin
      character*16 karta
      double precision difconf,difconf_simple
      real start,finish
      real rmsdtab(4,maxconf*(maxconf-1)/2)
      
      call initialize
      call getarg(1,pdbname)
      call getarg(2,karta)
      prefix=pdbname(:index(pdbname,'.')-1)
      outname=prefix(:len_trim(prefix))//'.'//'out_simplefit'
      open (iout,file=outname)
      call cpu_time(start)
      print *,"Opening ",pdbname(:len_trim(pdbname))
      write (iout,*) "Opening ",pdbname(:len_trim(pdbname))
      open(ipdbin,file=pdbname,status="old")
      do icon=1,maxconf
        call read_pdb(icon,*10,*20)
      enddo
   20 ncon=icon-1
      close(ipdbin)
      write (iout,*) "ncon",ncon
c      print *,"Opening ",pdb2
c      open(ipdbin,file=pdb2,status="old")
c      call read_pdb(2,*10)
c      close(ipdbin)
#ifdef DEBUG
      print *,"nchain",nchain
      do i=1,nchain
        do j=chain_border(1,i),chain_border(2,i)
          write (iout,'(a3,3i4,3f10.5,5x,3f10.5)') restyp(itype(j)),j,
     &     j-chain_border(1,i)+1,i,allcart(:,j,icon),allcart(:,j,jcon)
        enddo
        write (iout,*) 
      enddo
#endif
      call cpu_time(finish)
      write (iout,*) "Time to read data",finish-start," s"
      start=finish
      write (iout,'(3a10)') "Pairs","RMSD_s","RMSD_r"
      ii=0
      do icon=1,ncon-1
        do jcon=icon+1,ncon
          ii = ii+1
          rmsd_simple=difconf_simple(icon,jcon)
          rmsd=difconf(icon,jcon)
          write (iout,'(2i5,2f10.5)') icon,jcon,rmsd_simple,rmsd
        enddo
      enddo
      call cpu_time(finish)
      write (2,*) "CPU time rms",finish-start," s"
      stop
   10 stop "Error in reading pbd file(s)"
      end
c----------------------------------------------------------------------
      subroutine read_pdb(icon,*,*)
      implicit none
      integer i,j,k,ires
      include "DIMENSIONS"
      include "sizesclu.dat"
      include "COMMON.CONTROL"
      include "COMMON.CHAIN"
      include 'COMMON.NAMES'
      include "COMMON.CLUSTER"
      include "COMMON.IOUNITS"
      character*80 karta
      integer icon
      integer rescode

      if (icon.eq.1) then
      nchain=0
      chain_border(1,1)=1
      endif
      ires=0
      do while (ires.lt.maxres)
        read(ipdbin,'(a80)',end=103,err=102) karta
        if (index(karta,'ENDMDL').gt.0) goto 102
c        print *,karta
        if (karta(1:4).eq."ATOM".and.karta(14:15).eq."CA") then
          ires=ires+1
          ireschain(ires)=nchain+1
          itype(ires)=rescode(ires,karta(18:20),0)
          read (karta(31:54),'(3f8.3)',end=202,err=202)
     &          (allcart(j,ires,icon),j=1,3)
        else if (karta(1:3).eq."TER" .and. icon.eq.1) then
          nchain=nchain+1
          chain_border(2,nchain)=ires
          chain_border(1,nchain+1)=ires+1
          chain_length(nchain)=ires-chain_border(1,nchain)+1
        endif
      enddo
  102 continue
c      print *,"102 ",icon
      if (icon.gt.1.and.nres.ne.ires) then
        write (iout,*) "Numbers of residues is ",
     &  " different than in preceding structure(s)",ires,nres
        return1
      endif
      nres=ires
#ifdef DEBUG
      write (iout,*) " Conformation",icon," nchain",nchain
      write (iout,*) "chain_length",chain_length(:nchain)
#endif
c      print *,"chain_border"
c      do i=1,nchain
c        print *,chain_border(1,i),chain_border(2,i)
c      enddo
      if (icon.eq.1) then
      if (nstart.eq.0) nstart=1
      if (nend.eq.0) nend=nres
      nstart_sup=nstart
      nend_sup=nend
c      print *, "Finished broadcast coordinates"
      call chain_symmetry(nchain,nres,itype,chain_border,
     &  chain_length,npermchain,tabpermchain,nchain_group,nequiv,
     &  iequiv,mapchain)
      endif
#ifdef PRINT_PDB 
      k=0
      do i=1,nchain
        do j=chain_border(1,i),chain_border(2,i)
          write (iout,'(a3,3i4,3f10.5)') restyp(itype(j)),j,
     &     j-chain_border(1,i)+1,i,allcart(:,j,icon)
        enddo
        write (iout,*) 
      enddo
#endif
      return
  103 return2
  202 stop "Error in pdbfile"
      end 
c-------------------------------------------------------------------------
      double precision function difconf_simple(icon,jcon)
      implicit none
      include 'DIMENSIONS'
      include 'sizesclu.dat'
      include 'COMMON.CONTROL'
      include 'COMMON.CLUSTER'
      include 'COMMON.CHAIN'
      include 'COMMON.IOUNITS'
      integer listmin(maxchain)
      common /permutacja/listmin
      double precision przes(3),obrot(3,3)
      double precision rms
      integer icon,jcon,k,l
      character*3 liczba_i,liczba_j
c      write (iout,*) "DIFCONF: ICON",icon," JCON",jcon
      do k=1,nchain
        listmin(k)=k
      enddo
      do k=1,nres
        do l=1,3
          cref(l,k)=allcart(l,k,icon)
          c(l,k)=allcart(l,k,jcon)
        enddo
c        write (iout,'(i5,3f10.5,5x,3f10.5)') k,cref(:,k),c(:,k)
      enddo
      call rmscalc1(rms)
      difconf_simple=rms
      RETURN
      END
c-------------------------------------------------------------------------
      double precision function difconf(icon,jcon)
      implicit none
      include 'DIMENSIONS'
      include 'sizesclu.dat'
      include 'COMMON.CONTROL'
      include 'COMMON.CLUSTER'
      include 'COMMON.CHAIN'
      include 'COMMON.IOUNITS'
      integer ipermmin
      double precision przes(3),obrot(3,3)
      double precision rmscalc
      integer icon,jcon,k,l
      character*3 liczba_i,liczba_j
c      write (iout,*) "DIFCONF: ICON",icon," JCON",jcon
      do k=1,nres
        do l=1,3
          cref(l,k)=allcart(l,k,icon)
          c(l,k)=allcart(l,k,jcon)
        enddo
c        write (iout,'(i5,3f10.5,5x,3f10.5)') k,cref(:,k),c(:,k)
      enddo
      difconf=rmscalc(c(1,1),cref(1,1),przes,obrot,ipermmin)
#ifdef PDBOUT
      write (liczba_i,'(bz,i3.3)') icon
      write (liczba_j,'(bz,i3.3)') jcon
      pdbfile=prefix(:len_trim(prefix))//
     &   '_supf_'//liczba_i//'_'//liczba_j//'.pdb'
      open(igeom,file=pdbfile)
      write (igeom,'(a,i5)') "REMARK Structure",icon
      call pdbout(cref)
      call rotate(obrot,c,crefjlee)
      do k=1,nres
        crefjlee(:,k)=crefjlee(:,k)+przes(:)
      enddo
      write (igeom,'(a,i5)') "REMARK Structure",jcon
      call pdbout(crefjlee)
#endif
      RETURN
      END
